@@ -4,7 +4,7 @@
 #' parameters.
 #' 
 #' @usage \method{predict}{crm}(object,newdata=NULL,ddl=NULL,parameter=NULL,unique=TRUE,
-#'              vcv=FALSE,se=FALSE,chat=1,subset,select,...)
+#'                    vcv=FALSE,se=FALSE,chat=1,subset,select,...)
 #' @param object model object
 #' @param newdata a dataframe for crm 
 #' @param ddl list of dataframes for design data; created by call to
@@ -29,7 +29,7 @@
 #' dipper.proc=process.data(dipper,model="cjs",begin.time=1)
 #' dipper.ddl=make.design.data(dipper.proc)
 #' mod.Phisex.pdot=crm(dipper.proc,dipper.ddl,
-#'   model.parameters=list(Phi=list(formula=~sex+time),p=list(formula=~1)),hessian=TRUE)
+#'    model.parameters=list(Phi=list(formula=~sex+time),p=list(formula=~1)),hessian=TRUE)
 #' xx=predict(mod.Phisex.pdot)
 #' xx
 #' xx=predict(mod.Phisex.pdot,newdata=dipper[c(1,23),],vcv=TRUE)
@@ -45,8 +45,8 @@ predict.crm <-function(object,newdata=NULL,ddl=NULL,parameter=NULL,unique=TRUE,v
 			newdata.proc=process.data(newdata,model=object$model,begin.time=object$data$begin.time,groups=names(object$data$group.covariates),accumulate=FALSE)
 			ddl=make.design.data(newdata.proc,parameters=object$design.parameters)
 			dml=create.dml(ddl,model.parameters=object$model.parameters,design.parameters=object$design.parameters)
-		    object$results$model_data$Phi.dm=dml$Phi
-			object$results$model_data$p.dm=dml$p		
+		    object$results$model_data$Phi.dm=dml$Phi$fe
+			object$results$model_data$p.dm=dml$p$fe		
 		}else
 			stop("Invalid newdata")
 	}
@@ -63,4 +63,4 @@ predict.crm <-function(object,newdata=NULL,ddl=NULL,parameter=NULL,unique=TRUE,v
 		} else
 			return(compute.real(object,parameter,ddl,unique,vcv,se,chat,subset=substitute(subset),select))		
 	}
-}
\ No newline at end of file
+}
