@@ -1,9 +1,13 @@
 print.marked.version <- function()
 { library(help=marked)$info[[1]] -> version
 	version <- version[pmatch("Version",version)]
-	um <- strsplit(version," ")[[1]]
-	version <- um[nchar(um)>0][2]
-	hello <- paste("This is marked ",version,"\n",sep="")
+	if(!is.null(version))
+	{
+		um <- strsplit(version," ")[[1]]
+	    version <- um[nchar(um)>0][2]
+	    hello <- paste("This is marked ",version,"\n",sep="")
+	} else
+		hello <- "This is marked"
 	packageStartupMessage(hello)
 }
 .onAttach <- function(...) { 
